/******************************************************************************
 Keys.h

		Key data manager interface for the SignWriter key editor
		
		Copyright  1995 Center for Sutton Movement Writing. All rights reserved.

 ******************************************************************************/
 
#pragma once



#include "SymCode.h"
#include "KeyEditConsts.h"
#include "LongCoordinates.h"



		// Key limits

#define		KEY_SYMS	  5		// max symbols per key box
#define		KEY_VARS	 10		// symbol variations per key
#define		KEY_CODES	 50		// highest key code supported


	
		// Keyboard keys
		
#define		SIZ_ROW1	12		// 12 keys across top row
#define		SIZ_ROW2	12
#define		SIZ_ROW3	11
#define		SIZ_ROW4	10		// 10 keys across bottom row



		// Keyboard key codes
		
#define KEY_ROW1  { 0x12, 0x13, 0x14, 0x15, 0x17, 0x16,   \
					0x1A, 0x1C, 0x19, 0x1D, 0x1B, 0x18 }			// '1'..'='
									
#define KEY_ROW2  { 0x0C, 0x0D, 0x0E, 0x0F, 0x11, 0x10,   \
					0x20, 0x22, 0x1F, 0x23, 0x21, 0x1E } 			// 'Q'..']'

#define KEY_ROW3  { 0x00, 0x01, 0x02, 0x03, 0x05, 0x04,   \
					0x26, 0x28, 0x25, 0x29, 0x27       }			// 'A'..'''
									
#define KEY_ROW4  { 0x06, 0x07, 0x08, 0x09, 0x0B, 0x2D,   \
					0x2E, 0x2B, 0x2F, 0x2C			   } 			// 'Z'..'/'								



	  	// SignWriter symbol command key codes
	  	
#define		KEY_FILL		0x2D					// 'N'
#define		KEY_CURSOR		0x2E					// 'M'
#define		KEY_MIRROR		0x2B					// ','
#define		KEY_CHANGE		0x2F					// '.'
#define		KEY_ROTATE		0x2C					// '/'



		// Key box structure (pseudo-opaque type)
		
typedef struct {
	
	short	 descender;				// symbol descender	(fingerspelling only) 
	ObjType  selType;				// selection type (none, box, sym)
	short	 selSym;				// index of selected sym (if any)
	short	 numSyms;				// number of symbols in box
	SymCode	 sym[KEY_SYMS];			// symbols
	
} BoxRec, *Box;



		// Key variation structure (pseudo-opaque type)

typedef  Box  	 RowRec[KEY_VARS];
typedef  RowRec  *BoxRow;



	  // Sign key management
	  
void  NewSignKey    (short keyCode);	  
void  DisposSignKey (short keyCode);
	  
	  
	  
	  // Finger key management

void  NewFingerKey    (short keyCode);
void  DisposFingerKey (short keyCode);


	
	  // Clipboard management

void  DisposKey (ObjType type, void *rec);		// may be Box or BoxRow


						 
	  // Sign key editing				 				  

void  GetSignKey (short keyCode, BoxRow *rec);				  
void  SetSignKey (short keyCode, BoxRow rec);			

void  CopySignKey   (BoxRow rec, BoxRow *copyRec);
void  ClearSignKey  (BoxRow rec);				  	 			  	 		  

				
							 
	  // Finger key editing				 				  

void  GetFingerKey (short keyCode, Box *rec);				  
void  SetFingerKey (short keyCode, Box rec);			

void  CopyFingerKey   (Box rec, Box *copyRec);
void  ClearFingerKey  (Box rec);				  	 			  	 		  
							 
							 
							 
	  // Sign key variation editing				 				  

void  GetSignKeyVar (short keyCode, short keyVar, Box *rec);				  
void  SetSignKeyVar (short keyCode, short keyVar, Box rec);			

void  CopySignKeyVar   (Box rec, Box *copyRec);
void  ClearSignKeyVar  (Box rec);			
void  DisposSignKeyVar (Box rec);			



	  // Selection

void  GetSelectType (Box rec, ObjType *selType);
void  SetSelectType (Box rec, ObjType selType);

void  GetSelectSym (Box rec, short *symInx);
void  SetSelectSym (Box rec, short symInx);



	 // Fingerspelling descenders
 				 				  	
void  GetDescender (Box rec, short *descendVal);
void  SetDescender (Box rec, short descendVal);
  
  
  
	  // Symbol count
	  	  	
void  GetSymCount (Box rec, short *symCount);
void  SetSymCount (Box rec, short symCount);
	  
	  
	  
	  // Symbol editing

void  GetSymBase (Box rec, short symInx, Byte **symBase);
				  				  
void  CopySym   (Box src, short srcInx, Box dest);
void  DeleteSym (Box rec, short symInx);

	
	
	  // Sign key drawing
				 
void  DrawSignKey (Byte keyCode, Byte keyVar, BitMap *img, Rect *keyBox, Boolean thumbNail);
				   
void  DrawSignSelect  (Byte keyCode, Byte keyVar, BitMap *img, Rect *keyBox);				 
void  EraseSignSelect (BitMap *img);				 
	
	
	
	  // Finger key drawing
	 
void  DrawFingerKey (Byte keyCode, BitMap *img, Rect *keyBox, Boolean thumbNail);
		 
void  DrawFingerSelect  (Byte keyCode, BitMap *img, Rect *keyBox);  
void  EraseFingerSelect (BitMap *img);	  
	  
	  
	  
	  // File I/O support
	  
void  PackSignKey   (short keyCode, Handle keyBuf, short *keyBytes);
void  UnpackSignKey (short keyCode, Handle keyBuf, short  keyBytes);

void  PackFingerKey   (short keyCode, Handle keyBuf, short *keyBytes);
void  UnpackFingerKey (short keyCode, Handle keyBuf, short  keyBytes);

	  
			   