/****
 * CFingKeyWndPane.c
 *
 *	Fingerspelling key window pane methods for the SignWriter key editbor.
 *
 *  Copyright 1996 The Center for Sutton Movement Writing. All rights reserved.
 *
 ****/



#include "CFingKeyWndPane.h"
#include "CBureaucrat.h"
#include "CFileWndKeys.h"
#include "CFingKeyWndTask.h"
#include "CWindow.h"
#include "Events.h"
#include "Global.h"
#include "LongCoordinates.h"
#include "LongQD.h"
#include "KeyEditConsts.h"
#include "Symbols.h"
#include "TBUtilities.h"





/*** Global Variables ***/


extern short		 gClicks;			/* Single or double click */


extern void			*gClipRec;			/* Clipboard */
extern ObjType	 	 gClipType;


extern CFileWndKeys	*gKeyPanes;			/* File window keyboard */





/****************************  Initialization  ****************************/


/***
 * IFingKeyWndPane
 *
 ***/
 
void CFingKeyWndPane::IFingKeyWndPane (CView *anEnclosure, CBureaucrat *aSupervisor,
							   		   Byte keyCode)
{
	Box	 rec;
	
	
	itsKeyCode    = keyCode;		
	itsOn		  = FALSE;

	GetFingerKey(itsKeyCode, &rec);  
	SetSelectType(rec, none);
	
	inherited::IPane(anEnclosure, aSupervisor,
					 FKPANE_WIDTH, FKPANE_HEIGHT,							 
					 FKPANE_LEFT, FKPANE_TOP, 
					 sizFIXEDSTICKY, sizFIXEDSTICKY);
						 			
}

 	
 	
 	
/********************************  Drawing  ********************************/


/***
 * Draw (override)
 *
 *  Draw the key contents.
 *
 ***/
 
void CFingKeyWndPane::Draw (Rect *area)
 
{ 
	UpdateKey();
}



/***
 * UpdateKey
 *
 *  Update the key after editing.
 *
 ***/

void CFingKeyWndPane::UpdateKey (void)
 
{
	LongRect lkeyBox;
	Rect	 keyBox;
	Box		 rec;
	ObjType  selType;
	
	
	// Draw key contents
	
	Prepare();
	
	GetFrame(&lkeyBox);
	LongToQDRect(&lkeyBox, &keyBox);

	EraseRect(&keyBox);	
	DrawFingerKey(itsKeyCode, &thePort->portBits, &keyBox, FALSE);	


	// update select state

	GetFingerKey(itsKeyCode, &rec);
	GetSelectType(rec, &selType);	
	
	if (selType == none)								// show descender
		ToggleDescender();	
		
	if (selType == box)									// show box
		ToggleBox();				
	else												
		itsOn = FALSE;									// blink is off
	
}



/***
 * ToggleBox
 *
 *  Invert key box.
 *
 ***/

void CFingKeyWndPane::ToggleBox (void)
 
{	
	LongRect  keyBox;
	
	
	GetFrame(&keyBox);
		
	Prepare();
	LInvertRect(&keyBox);
			
}


 
/***
 * ToggleCursor
 *
 *  Draw/erase insertion point.
 *
 ***/
 
void CFingKeyWndPane::ToggleCursor (void)

{
	LongRect keyBox;
	
	
	GetFrame(&keyBox);
		
	Prepare();
	PenMode(patXor);
	PenPat(black);

	LMoveTo(keyBox.left + KCURS_LEFT, keyBox.top + KCURS_TOP);
	Line(0, KCURS_LEN);
	
	itsOn = !itsOn;
	
}



/***
 * ToggleSelect
 *
 *  Draw/erase selected symbol.
 *
 ***/
 
void CFingKeyWndPane::ToggleSelect (void)

{
	LongRect lkeyBox;
	Rect	 keyBox;
	
			
	GetFrame(&lkeyBox);
	LongToQDRect(&lkeyBox, &keyBox);
	
	Prepare();
	
	if (!itsOn)
		DrawFingerSelect(itsKeyCode, &thePort->portBits, &keyBox);
		
	else
	 	EraseFingerSelect(&thePort->portBits);			
	 		
	itsOn = !itsOn;
	
}

 
 
/***
 * ToggleDescender
 *
 *  Draw/erase descender line.
 *
 ***/
 
void CFingKeyWndPane::ToggleDescender (void)

{
	Box	   rec;
	short  descend;
	
	
	GetFingerKey(itsKeyCode, &rec);
	GetDescender(rec, &descend);	
		
	Prepare();
	PenMode(patXor);
	PenPat(gray);
	
	LMoveTo(0, descend);
	LLineTo(FKPANE_WIDTH, descend);
	
}

	


/********************************  Selection  ********************************/


/***
 * BlinkSelect 
 *
 *  Blink the insertion point or
 *  currently selected symbol.
 *
 ***/
 
void CFingKeyWndPane::BlinkSelect (long *maxSleep)

{	
	Box		 rec;
	ObjType  selType;
	
	
	GetFingerKey(itsKeyCode, &rec);				  
	GetSelectType(rec, &selType);	
	
	switch (selType) {

		case box:
					
			*maxSleep = GetCaretTime();
			
			// boxes don't blink
			
			break;
			
			
		case sym:
						
			*maxSleep = (itsOn) ? SYM_OFF : SYM_ON;

			ToggleSelect();
		
			break;
			
		
		case none:
			
			*maxSleep = GetCaretTime();
			
			ToggleCursor();
			
			break;
		
	}
			
}



/***
 * Deselect
 *
 *  Deselect the key box and
 *  update the display accordingly.
 *
 ***/
 
void CFingKeyWndPane::Deselect (void)

{
	Box		 rec;
	ObjType  selType;
	Rect  	 keyBox;


	GetFingerKey(itsKeyCode, &rec);				  
	GetSelectType(rec, &selType);
	
		
	switch (selType) {
	
		case box:
		
			ToggleBox();								// de-invert box
			break;
			

		case sym:
			
			if (!itsOn) ToggleSelect();					// draw selected sym					
			break;
			
			
		case none:

			if (itsOn) ToggleCursor();					// turn off cursor
			break;		
				
	}
	
	
	SetSelectType(rec, none);							// do deselect
	
	
	if (selType != none)								// show descender
		ToggleDescender();								
	
	itsOn = FALSE;										// blink is off

}



/***
 * Deactivate (override default)
 *
 *  Deselect before leaving keybox window.
 *
 ***/
 
void CFingKeyWndPane::Deactivate (void)
 
{
	Deselect();
	
	inherited::Deactivate();							// do inherited
	
}



/***
 * Close
 *
 *  Deselect before closing keybox window.
 *
 ***/
 
void CFingKeyWndPane::Close (void)
 
{
	Deselect();
}

	

/***
 * SelectionType
 *
 *  Return the type of the object currently
 *  selected in the finger key window box.
 *
 ***/
 
ObjType CFingKeyWndPane::SelectionType (void)

{
	Box		 rec;
	ObjType  selType;
	
	
	GetFingerKey(itsKeyCode, &rec);				  
	GetSelectType(rec, &selType);
	
	return selType;
	
}




/******************************  Edit commands  *****************************/

	
/***
/* 
/*  DoCopy
/*
/*  Copy selected item to the clipboard. Not undoable.
/*
 ***/
		
void CFingKeyWndPane::DoCopy (void)

{
	Box	 rec;
	
		
	if (gClipRec != NULL)
		DisposKey(gClipType, gClipRec);
		
	GetFingerKey(itsKeyCode, &rec);	
	CopyFingerKey(rec, (Box *) &gClipRec);

	GetSelectType(rec, &gClipType);		
	// CFingKeyWindow::UpdateMenu ensures this is 'box' or 'sym'		
	
}
	


/***
/* 
/*  DoClear
/*
/*  Erase the currently selected item
/*
 ***/

void CFingKeyWndPane::DoClear (short taskIndex)

{
	CFingKeyWndClear *theClearTask;

						
	theClearTask = new CFingKeyWndClear;
	theClearTask->IFingKeyWndClear(taskIndex, this, itsKeyCode);
	theClearTask->Do();
	
	itsSupervisor->Notify(theClearTask);
	itsLastTask = theClearTask;
	
}



/***
/* 
/*  DoPaste
/*
/*  Paste key record from the clipboard.
/*
 ***/

void CFingKeyWndPane::DoPaste (void)

{
	CFingKeyWndPaste *thePasteTask;

						
	thePasteTask = new CFingKeyWndPaste;
	thePasteTask->IFingKeyWndPaste(this, itsKeyCode);
	thePasteTask->Do();

	itsSupervisor->Notify(thePasteTask);
	itsLastTask = thePasteTask;
	
}	


	
/***
/* 
/*  DoAdd
/*
/*  Add new symbol from the symbol window.
/*
 ***/
		
void CFingKeyWndPane::DoAdd (void)

{
	CFingKeyWndAdd *theAddTask;

						
	theAddTask = new CFingKeyWndAdd;
	theAddTask->IFingKeyWndAdd(this, itsKeyCode);
	theAddTask->Do();

	itsSupervisor->Notify(theAddTask);
	itsLastTask = theAddTask;
	
}
	
	


/*****************************  Symbol commands  *****************************/



// Symbol state

static Byte	  *symBase;			// symbol base address
static short   symCode;			// symbol code 		
static short   symVar;			// sym variation
static short   symFill;			// fill variation
static short   symRot;			// rotation value
static short   symFlop;			// flop value
static short   symMark;			// mark value
static short   symXPos;			// X position
static short   symYPos;			// Y position



	
/***
/* 
/*  Setup
/*
/*  Do preliminary work for symbol modification methods.
/*
 ***/

void CFingKeyWndPane::Setup (void)

{
	short	selInx;
	Box		key;
	
	
	GetFingerKey(itsKeyCode, &key);			// get initial sym state				  	
	GetSelectSym(key, &selInx);
	GetSymBase(key, selInx, &symBase);

	GetSymInfo(symBase, &symCode, &symVar, &symFill, &symRot, &symFlop, 
									 	  &symMark, &symXPos, &symYPos);

}



/***
/* 
/*  Update
/*
/*  Update modified symbol. The symbol state
/*  has been changed by one of the symbol
/*  command methods listed below.  (Dawdle 
/*  method redraws the modified symbol.)
/*
 ***/

void CFingKeyWndPane::Update (void)

{
	LongPt	framePt, junk;
	Box		key;

								 
	GetCoords(symCode, symVar, symFill, symRot, symFlop, 		// new symbol state
							&framePt, &junk, FALSE); 	
														
	symXPos = (FKPANE_WIDTH  - framePt.h) / 2;					// center sym in box				
	symYPos = (FKPANE_HEIGHT - framePt.v) / 2;						
													
																	
	SetSymInfo(symBase, symCode, symVar, symFill, symRot, 		// update sym code	
					symFlop, symMark, symXPos, symYPos);
								
								
	GetFingerKey(itsKeyCode, &key);								// update descender
	SetDescender(key, symYPos + framePt.v);

}


	
/***
/* 
/*  DoChange
/*
/*  Change the symbol variation.
/*
 ***/

void CFingKeyWndPane::DoChange (void)

{
	short  maxVar;
	
	
	Setup();
	  
	GetSymVars(symCode, &maxVar);		 
	
	if (KeyIsDown(SHFT_KEY)) {
	
		symVar--;
		if (symVar < 0)
			symVar = maxVar;
			
	} else {
	
		symVar++;
		if (symVar > maxVar)
			symVar = 0;
	}
	
	Update();
	
}



/***
/* 
/*  DoFill
/*
/*  Change the fill variation.
/*
 ***/
 
void CFingKeyWndPane::DoFill (void)

{
	short  maxFill;
	
	
	Setup();
	  
	GetFillVars(symCode, symVar, &maxFill);		 
	
	if (KeyIsDown(SHFT_KEY)) {
	
		symFill--;
		if (symFill < 0)
			symFill = maxFill;
			
	} else {
	
		symFill++;
		if (symFill > maxFill)
			symFill = 0;
	}
	
	Update();
	
}
 	
 	
 		
/***
/* 
/*  DoMirror
/*
/*  Change symbol to its mirror image.
/*
 ***/
 
void CFingKeyWndPane::DoMirror (void)

{
	ImageClass	imgClass;
	short		numRots;
	short  		maxRot, maxFlop;
	
	
	Setup();
	  
	GetAttrs(symCode, symVar, symFill, &imgClass, &maxRot, &maxFlop);

	numRots = maxRot + 1;
			
	if (numRots > 3) {
	
		if (imgClass == class3B)
			symRot += 4;
			
		// magic mirror formula - it works!
		symRot = (numRots - (symRot % numRots)) % numRots;
	}
		
	if (maxFlop == 1)
		symFlop = (symFlop + 1) % 2;
	
	Update();
	
}



/***
/* 
/*  DoRotate
/*
/*  Change the symbol rotation.
/*
 ***/
 
void CFingKeyWndPane::DoRotate (void)

{
	ImageClass	junk1;
	short  		maxRot, junk2;
	
	
	Setup();
	  
	GetAttrs(symCode, symVar, symFill, &junk1, &maxRot, &junk2);

	if (!KeyIsDown(SHFT_KEY)) {
	
		symRot++;
		if (symRot > maxRot)
			symRot = 0;
			
	} else {
	
		symRot--;
		if (symRot < 0)
			symRot = maxRot;
	}
			
	Update();
			
}	
 		


/***
/* 
/*  DoCursor
/*
/*  Deselect symbol and redisplay cursor.
/*
 ***/

void CFingKeyWndPane::DoCursor (void)

{
	Box	 key;
	
	
	GetFingerKey(itsKeyCode, &key);				  
	SetSelectType(key, none);

	UpdateKey();
	
}	



/***
 * DoSymCommand
 *
 *	Perform symbol edit command that corresponds
 *  to the specified key code.
 *
 ***/
 
void CFingKeyWndPane::DoSymCommand (Byte keyCode)
 
{
	if (SelectionType() == sym)				// sym must be selected
	
	 	switch (keyCode) {
	 	
	 		case KEY_FILL:
	 		
	 			DoFill(); 			
	 			break;
	 		
	 		
	 		case KEY_CURSOR:
	 		
	 			DoCursor(); 			
	 			break;
	 		
	 		
	 		case KEY_MIRROR:
	 		
	 			DoMirror(); 			
	 			break;
	 			
	 	
	 		case KEY_CHANGE:
	 		
	 			DoChange(); 			
	 			break;
	 			
	 			
	 		case KEY_ROTATE:
	 		
	 			DoRotate(); 			
	 			break;
	 		
		}
		
}




/**********************************  Mouse  ***********************************/
	
	
/***
 * DoClick (override default method)
 *
 *  If hit point is on a symbol, select it.
 *  If hit point is on two or more symbols,
 *  select the smallest.  If mouse is dragged,
 *  move selected symbol accordingly.
 *
 ***/ 

void CFingKeyWndPane::DoClick (Point hitPt, short modifierKeys, long when)

{
	CFingKeyWndDrag  *theDragTask;
	CFingKeyWndDesc  *theDescenderTask;

	short	  symCount, junk;
	short     symInx, selInx, selSym;
	short	  symSize, minSize;
	LongPt    framePt, notUsed;
	LongPt	  trackStart, trackEnd;
	Point	  keyPt;
	Rect      symFrame;
	LongRect  boxFrame;
	ObjType	  selType;
	Byte	  *symBase;
	Box		  key;
	
	
	Deselect();												// zap old selection


	GetFingerKey(itsKeyCode, &key);							// get symbol count
	GetSymCount(key, &symCount);	

	
	GetFrame(&boxFrame);									// hit pt -> key coords	
	keyPt.h = hitPt.h - boxFrame.left;
	keyPt.v = hitPt.v - boxFrame.top;
	
	
	minSize = 10000;										// look for symbol hit...
	selInx  = -1;

	for (symInx = 0; symInx < symCount; symInx++) {	
	
	
		GetSymBase(key, symInx, &symBase);					// get sym image frame

		GetSymInfo(symBase, &symCode, &symVar, &symFill, &symRot, &symFlop, 
							&junk, &symXPos, &symYPos);
		
		GetCoords(symCode, symVar, symFill, symRot, symFlop, 
								&framePt, &notUsed, FALSE); 	
								
		SetRect(&symFrame, 0, 0, framePt.h, framePt.v);
		OffsetRect(&symFrame, symXPos, symYPos);
		
		
		if (PtInRect(keyPt, &symFrame)) {					// hit a symbol
		
			symSize = (symFrame.bottom - symFrame.top) *
					  (symFrame.right  - symFrame.left);
			
			if (symSize < minSize) {	
				minSize = symSize;
				selInx = symInx;
			}
				
		}
	}
	
	
	if (selInx >= 0) {										// symbol selected...
	
	
		SetSelectType(key, sym);							
		SetSelectSym(key, selInx);
		
		
		UpdateKey();										// erase symbol
		
					
		QDToLongPt(hitPt, &trackStart);						// track mouse
	
		theDragTask = new CFingKeyWndDrag;
		theDragTask->IFingKeyWndDrag(this, key, selInx);

		ForceNextPrepare();
		Prepare();
		
		TrackMouse(theDragTask, &trackStart, &boxFrame);
		
		theDragTask->Dispose();
		
	}

	else {													// no symbol hit...
	
							
		QDToLongPt(hitPt, &trackStart);
		
		theDescenderTask = new CFingKeyWndDesc;				// track descender drag
		theDescenderTask->IFingKeyWndDesc(this, key);

		ForceNextPrepare();
		Prepare();
		
		TrackMouse(theDescenderTask, &trackStart, &boxFrame);
		
		theDescenderTask->Dispose();
		
	}
	

	if (gClicks == 2) {										// double click....
				
		SetSelectType(key, box);
				
		UpdateKey();										// select box
	
	}
		
}				
	
